import os

import dotenv
import nonebot

from .defines import *


def auto_set_env(config: dict):
    dotenv.load_dotenv(".env")
    if os.getenv("DRIVER", None) is not None:
        nonebot.logger.info("Driver already set in environment variable, skip auto configure.")
        return
    if config.get("satori", {'enable': False}).get("enable", False):
        os.environ["DRIVER"] = get_driver_string(ASGI_DRIVER, HTTPX_DRIVER, WEBSOCKETS_DRIVER)
        nonebot.logger.info("Enable Satori, set driver to ASGI+HTTPX+WEBSOCKETS")
    else:
        os.environ["DRIVER"] = get_driver_string(ASGI_DRIVER)
        nonebot.logger.info("Disable Satori, set driver to ASGI")
    return



import json
import os

import nonebot
from nonebot.adapters import satori


def init(config: dict):
    if config.get("satori", None) is None:
        nonebot.logger.info("Satori config not found, skip Satori init.")
        return None
    satori_config = config.get("satori")
    if not satori_config.get("enable", False):
        nonebot.logger.info("Satori not enabled, skip Satori init.")
        return None
    if os.getenv("SATORI_CLIENTS", None) is not None:
        nonebot.logger.info("Satori clients already set in environment variable, skip.")
    os.environ["SATORI_CLIENTS"] = json.dumps(satori_config.get("hosts", []), ensure_ascii=False)
    config['satori_clients'] = satori_config.get("hosts", [])
    return


def register():
    if os.getenv("SATORI_CLIENTS", None) is not None:
        driver = nonebot.get_driver()
        driver.register_adapter(satori.Adapter)



import nonebot
from nonebot.adapters.onebot import v11, v12


def init():
    pass


def register():
    driver = nonebot.get_driver()
    driver.register_adapter(v11.Adapter)
    driver.register_adapter(v12.Adapter)



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/11 下午5:24
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : __init__.py.py
@Software: PyCharm
"""

import nonebot
from liteyuki.utils import IS_MAIN_PROCESS
from liteyuki.plugin import PluginMetadata
from .nb_utils import adapter_manager, driver_manager

__plugin_meta__ = PluginMetadata(
    name="NoneBot2启动器",
)


def nb_run(*args, **kwargs):
    """
    初始化NoneBot并运行在子进程
    Args:
        **kwargs:

    Returns:
    """
    # 给子进程传递通道对象

    kwargs.update(kwargs.get("nonebot", {}))  # nonebot配置优先
    nonebot.init(**kwargs)

    driver_manager.init(config=kwargs)
    adapter_manager.init(kwargs)
    adapter_manager.register()

    try:
        nonebot.load_plugin("nonebot-plugin-lnpm")  # 尝试加载轻雪NoneBot插件加载器（Nonebot插件）
    except Exception as e:
        pass

    nonebot.run()


if IS_MAIN_PROCESS:
    from .dev_reloader import *

    liteyuki = get_bot()


    @liteyuki.on_before_start
    async def start_run_nonebot():
        liteyuki.process_manager.add_target(name="nonebot", target=nb_run, args=(), kwargs=liteyuki.config)



# -*- coding: utf-8 -*-
"""
NoneBot 开发环境重载监视器
"""
import os.path

from liteyuki.dev import observer
from liteyuki import get_bot, logger
from liteyuki.utils import IS_MAIN_PROCESS
from watchdog.events import FileSystemEvent


liteyuki = get_bot()

exclude_extensions = (".pyc", ".pyo")


@observer.on_file_system_event(
    directories=("src/nonebot_plugins",),
    event_filter=lambda event: not event.src_path.endswith(exclude_extensions) and ("__pycache__" not in event.src_path ) and os.path.isfile(event.src_path)
)
def restart_nonebot_process(event: FileSystemEvent):
    logger.debug(f"File {event.src_path} changed, reloading nonebot...")
    liteyuki.restart_process("nonebot")


ASGI_DRIVER = "~fastapi"
HTTPX_DRIVER = "~httpx"
WEBSOCKETS_DRIVER = "~websockets"


def get_driver_string(*argv):
    output_string = ""
    if ASGI_DRIVER in argv:
        output_string += ASGI_DRIVER
    for arg in argv:
        if arg != ASGI_DRIVER:
            output_string = f"{output_string}+{arg}"
    return output_string


def get_driver_full_string(*argv):
    return f"DRIVER={get_driver_string(argv)}"
